@@ -19,18 +19,21 @@
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
 
+from __future__ import division, absolute_import, print_function, unicode_literals
+from awlsim.core.compat import *
+
 from awlsim.coreserver.server import *
 from awlsim.coreserver.messages import *
 
 import sys
 import socket
 import errno
+import time
 
 
 class AwlSimClient(object):
 	def __init__(self):
 		self.serverProcess = None
-		self.socket = None
 		self.transceiver = None
 
 	def spawnServer(self, interpreter=None,
@@ -63,8 +66,7 @@ class AwlSimClient(object):
 			(host, port))
 		try:
 			family, socktype, proto, canonname, sockaddr =\
-				socket.getaddrinfo(host, port, socket.AF_INET,
-						   0, socket.SOL_TCP)[0]
+				socket.getaddrinfo(host, port, socket.AF_INET, socket.SOCK_STREAM)[0]
 			sock = socket.socket(family, socktype)
 			count = 0
 			while 1:
@@ -77,7 +79,12 @@ class AwlSimClient(object):
 							raise AwlSimError("Timeout connecting "
 								"to AwlSimServer %s (port %d)" %\
 								(host, port))
-						time.sleep(0.1)
+						self.sleep(0.1)
+						continue
+					if isJython and\
+					   e.strerror.endswith("java.nio.channels.CancelledKeyException"):
+						# XXX Jython workaround: Ignore this exception
+						printInfo("Warning: Jython connect workaround")
 						continue
 					raise
 				break
@@ -85,7 +92,6 @@ class AwlSimClient(object):
 			raise AwlSimError("Failed to connect to AwlSimServer %s (port %d): %s" %\
 				(host, port, str(e)))
 		printInfo("AwlSimClient: Connected.")
-		self.socket = sock
 		self.transceiver = AwlSimMessageTransceiver(sock)
 		self.lastRxMsg = None
 
@@ -107,19 +113,19 @@ class AwlSimClient(object):
 	def shutdown(self):
 		"""Shutdown all sockets and spawned processes."""
 
-		if self.socket:
-			try:
-				self.socket.shutdown(socket.SHUT_RDWR)
-			except socket.error as e:
-				pass
-			try:
-				self.socket.close()
-			except socket.error as e:
-				pass
-			self.socket = None
+		if self.transceiver:
+			self.transceiver.shutdown()
+			self.transceiver = None
 		if self.serverProcess:
-			self.serverProcess.terminate()
-			self.serverProcess.wait()
+			try:
+				self.serverProcess.terminate()
+				self.serverProcess.wait()
+			except NameError:
+				# XXX: Workaround: Jython currently does not implement terminate
+				if not isJython:
+					raise
+				printInfo("AwlSimClient: Jython Popen.terminate workaround: "
+					  "Not terminating server.")
 			self.serverProcess = None
 
 	def __rx_NOP(self, msg):
@@ -172,6 +178,9 @@ class AwlSimClient(object):
 	}
 
 	def processMessages(self, blocking=False):
+		self.lastRxMsg = None
+		if not self.transceiver:
+			return
 		try:
 			if blocking:
 				msg = self.transceiver.receiveBlocking()
@@ -179,7 +188,7 @@ class AwlSimClient(object):
 				msg = self.transceiver.receive()
 		except socket.error as e:
 			if e.errno == errno.EAGAIN:
-				return None
+				return
 			host, port = self.socket.getpeername()
 			raise AwlSimError("AwlSimClient: "
 				"I/O error in connection to server '%s (port %d)':\n%s" %\
@@ -200,25 +209,31 @@ class AwlSimClient(object):
 				"message 0x%02X" % msg.msgId)
 		handler(self, msg)
 
-	def __sendAndWait(self, txMsg, checkRxMsg, waitTimeoutMs=3000):
+	def sleep(self, seconds):
+		time.sleep(seconds)
+
+	def __sendAndWait(self, txMsg, checkRxMsg, waitTimeout=3.0):
 		self.transceiver.send(txMsg)
-		count = 0
-		while count < waitTimeoutMs:
+		now = monotonic_time()
+		end = now + waitTimeout
+		while now < end:
 			self.processMessages()
 			rxMsg = self.lastRxMsg
-			if rxMsg and checkRxMsg(rxMsg):
-				self.lastRxMsg = None
-				return rxMsg
-			time.sleep(0.01)
-			count += 10
+			if rxMsg:
+				if checkRxMsg(rxMsg):
+					return rxMsg
+			else:
+				# Queue is empty.
+				pass#XXX self.sleep(0.01)
+			now = monotonic_time()
 		raise AwlSimError("AwlSimClient: Timeout waiting for server reply.")
 
-	def __sendAndWaitFor_REPLY(self, msg, timeoutMs=3000):
+	def __sendAndWaitFor_REPLY(self, msg, timeout=3.0):
 		def checkRxMsg(rxMsg):
 			return rxMsg.msgId == AwlSimMessage.MSG_ID_REPLY and\
 			       rxMsg.inReplyToId == msg.msgId and\
 			       rxMsg.inReplyToSeq == msg.seq
-		return self.__sendAndWait(msg, checkRxMsg, timeoutMs).status
+		return self.__sendAndWait(msg, checkRxMsg, timeout).status
 
 	def reset(self):
 		msg = AwlSimMessage_RESET()
@@ -227,6 +242,8 @@ class AwlSimClient(object):
 			raise AwlSimError("AwlSimClient: Failed to reset CPU")
 
 	def setRunState(self, run=True):
+		if not self.transceiver:
+			return False
 		if run:
 			runState = AwlSimMessage_RUNSTATE.STATE_RUN
 		else:
@@ -235,12 +252,16 @@ class AwlSimClient(object):
 		status = self.__sendAndWaitFor_REPLY(msg)
 		if status != AwlSimMessage_REPLY.STAT_OK:
 			raise AwlSimError("AwlSimClient: Failed to set run state")
+		return True
 
 	def loadCode(self, code):
+		if not self.transceiver:
+			return False
 		msg = AwlSimMessage_LOAD_CODE(code)
-		status = self.__sendAndWaitFor_REPLY(msg, 10000)
+		status = self.__sendAndWaitFor_REPLY(msg, 10.0)
 		if status != AwlSimMessage_REPLY.STAT_OK:
 			raise AwlSimError("AwlSimClient: Failed to load code")
+		return True
 
 	def loadSymbolTable(self, symTabText):
 		msg = AwlSimMessage_LOAD_SYMTAB(symTabText)
@@ -249,42 +270,56 @@ class AwlSimClient(object):
 			raise AwlSimError("AwlSimClient: Failed to load symbol table")
 
 	def loadHardwareModule(self, name, parameters={}):
+		if not self.transceiver:
+			return False
 		msg = AwlSimMessage_LOAD_HW(name = name,
 					    paramDict = parameters)
 		status = self.__sendAndWaitFor_REPLY(msg)
 		if status != AwlSimMessage_REPLY.STAT_OK:
 			raise AwlSimError("AwlSimClient: Failed to load hardware module")
+		return True
 
 	def __setOption(self, name, value):
+		if not self.transceiver:
+			return False
 		msg = AwlSimMessage_SET_OPT(name, str(value))
 		status = self.__sendAndWaitFor_REPLY(msg)
 		if status != AwlSimMessage_REPLY.STAT_OK:
 			raise AwlSimError("AwlSimClient: Failed to set option '%s'" % name)
+		return True
 
 	def setLoglevel(self, level=Logging.LOG_INFO):
 		Logging.setLoglevel(level)
-		self.__setOption("loglevel", int(level))
+		return self.__setOption("loglevel", int(level))
 
 	def enableOBTempPresets(self, enable=True):
-		self.__setOption("ob_temp_presets", int(bool(enable)))
+		return self.__setOption("ob_temp_presets", int(bool(enable)))
 
 	def enableExtendedInsns(self, enable=True):
-		self.__setOption("extended_insns", int(bool(enable)))
+		return self.__setOption("extended_insns", int(bool(enable)))
 
 	def setPeriodicDumpInterval(self, interval=0):
-		self.__setOption("periodic_dump_int", int(interval))
+		return self.__setOption("periodic_dump_int", int(interval))
+
+	def setCycleTimeLimit(self, seconds=5.0):
+		return self.__setOption("cycle_time_limit", float(seconds))
 
 	def getCpuSpecs(self):
+		if not self.transceiver:
+			return None
 		msg = AwlSimMessage_GET_CPUSPECS()
 		rxMsg = self.__sendAndWait(msg,
 			lambda rxMsg: rxMsg.msgId == AwlSimMessage.MSG_ID_CPUSPECS)
 		return rxMsg.cpuspecs
 
 	def setCpuSpecs(self, cpuspecs):
+		if not self.transceiver:
+			return False
 		msg = AwlSimMessage_CPUSPECS(cpuspecs)
 		status = self.__sendAndWaitFor_REPLY(msg)
 		if status != AwlSimMessage_REPLY.STAT_OK:
 			raise AwlSimError("AwlSimClient: Failed to set cpuspecs")
+		return True
 
 	# Set the memory areas we are interested in receiving
 	# dumps for, in the server.
@@ -294,22 +329,32 @@ class AwlSimClient(object):
 	#  - repeat on n'th every cycle (repetitionFactor=n)
 	# If sync is true, wait for a reply from the server.
 	def setMemoryReadRequests(self, memAreas, repetitionFactor=0, sync=False):
-		self.transceiver.send(
-			AwlSimMessage_REQ_MEMORY(0, repetitionFactor, memAreas)
-		)
+		if not self.transceiver:
+			return False
+		msg = AwlSimMessage_REQ_MEMORY(0, repetitionFactor, memAreas)
 		if sync:
+			msg.flags |= msg.FLG_SYNC
 			status = self.__sendAndWaitFor_REPLY(msg)
 			if status != AwlSimMessage_REPLY.STAT_OK:
-				raise AwlSimError("AwlSimClient: Failed to set memory read reqs")
+				raise AwlSimError("AwlSimClient: Failed to set "
+					"memory read reqs")
+		else:
+			self.transceiver.send(msg)
+		return True
 
 	# Write memory areas in the server.
 	# memAreas is a list of MemoryAreaData instances.
 	# If sync is true, wait for a reply from the server.
 	def writeMemory(self, memAreas, sync=False):
-		self.transceiver.send(
-			AwlSimMessage_MEMORY(0, memAreas)
-		)
+		if not self.transceiver:
+			return False
+		msg = AwlSimMessage_MEMORY(0, memAreas)
 		if sync:
+			msg.flags |= msg.FLG_SYNC
 			status = self.__sendAndWaitFor_REPLY(msg)
 			if status != AwlSimMessage_REPLY.STAT_OK:
-				raise AwlSimError("AwlSimClient: Failed to write to memory")
+				raise AwlSimError("AwlSimClient: Failed to write "
+					"to memory")
+		else:
+			self.transceiver.send(msg)
+		return True
