@@ -19,9 +19,12 @@
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
 
+from __future__ import division, absolute_import, print_function, unicode_literals
+from awlsim.core.compat import *
+
 from awlsim.coreserver.memarea import *
-from awlsim.util import *
-from awlsim.cpuspecs import *
+from awlsim.core.util import *
+from awlsim.core.cpuspecs import *
 
 import struct
 import socket
@@ -39,31 +42,31 @@ class AwlSimMessage(object):
 	#	Reserved (16 bit)
 	#	Payload length (32 bit)
 	#	Payload (optional)
-	hdrStruct = struct.Struct(">HHHHI")
+	hdrStruct = struct.Struct(str(">HHHHI"))
 
 	HDR_MAGIC		= 0x5710
 	HDR_LENGTH		= hdrStruct.size
 
-	enum.start
-	MSG_ID_REPLY		= enum.item # Generic status reply
-	MSG_ID_EXCEPTION	= enum.item
-	MSG_ID_PING		= enum.item
-	MSG_ID_PONG		= enum.item
-	MSG_ID_RESET		= enum.item
-	MSG_ID_RUNSTATE		= enum.item
-	MSG_ID_LOAD_SYMTAB	= enum.item
-	MSG_ID_LOAD_CODE	= enum.item
-	MSG_ID_LOAD_HW		= enum.item
-	MSG_ID_SET_OPT		= enum.item
-	MSG_ID_CPUDUMP		= enum.item
-	MSG_ID_MAINTREQ		= enum.item
-	MSG_ID_GET_CPUSPECS	= enum.item
-	MSG_ID_CPUSPECS		= enum.item
-	MSG_ID_REQ_MEMORY	= enum.item
-	MSG_ID_MEMORY		= enum.item
-	enum.end
+	EnumGen.start
+	MSG_ID_REPLY		= EnumGen.item # Generic status reply
+	MSG_ID_EXCEPTION	= EnumGen.item
+	MSG_ID_PING		= EnumGen.item
+	MSG_ID_PONG		= EnumGen.item
+	MSG_ID_RESET		= EnumGen.item
+	MSG_ID_RUNSTATE		= EnumGen.item
+	MSG_ID_LOAD_SYMTAB	= EnumGen.item
+	MSG_ID_LOAD_CODE	= EnumGen.item
+	MSG_ID_LOAD_HW		= EnumGen.item
+	MSG_ID_SET_OPT		= EnumGen.item
+	MSG_ID_CPUDUMP		= EnumGen.item
+	MSG_ID_MAINTREQ		= EnumGen.item
+	MSG_ID_GET_CPUSPECS	= EnumGen.item
+	MSG_ID_CPUSPECS		= EnumGen.item
+	MSG_ID_REQ_MEMORY	= EnumGen.item
+	MSG_ID_MEMORY		= EnumGen.item
+	EnumGen.end
 
-	_strLenStruct = struct.Struct(">H")
+	_strLenStruct = struct.Struct(str(">H"))
 
 	@classmethod
 	def packString(cls, string):
@@ -102,12 +105,12 @@ class AwlSimMessage(object):
 		return cls()
 
 class AwlSimMessage_REPLY(AwlSimMessage):
-	enum.start
-	STAT_OK		= enum.item
-	STAT_FAIL	= enum.item
-	enum.end
+	EnumGen.start
+	STAT_OK		= EnumGen.item
+	STAT_FAIL	= EnumGen.item
+	EnumGen.end
 
-	plStruct = struct.Struct(">HHH")
+	plStruct = struct.Struct(str(">HHH"))
 
 	@classmethod
 	def make(cls, inReplyToMsg, status):
@@ -147,12 +150,12 @@ class AwlSimMessage_RESET(AwlSimMessage):
 		AwlSimMessage.__init__(self, AwlSimMessage.MSG_ID_RESET)
 
 class AwlSimMessage_RUNSTATE(AwlSimMessage):
-	enum.start
-	STATE_STOP	= enum.item
-	STATE_RUN	= enum.item
-	enum.end
+	EnumGen.start
+	STATE_STOP	= EnumGen.item
+	STATE_RUN	= EnumGen.item
+	EnumGen.end
 
-	plStruct = struct.Struct(">H")
+	plStruct = struct.Struct(str(">H"))
 
 	def __init__(self, runState):
 		AwlSimMessage.__init__(self, AwlSimMessage.MSG_ID_RUNSTATE)
@@ -282,6 +285,12 @@ class AwlSimMessage_SET_OPT(AwlSimMessage):
 		except ValueError as e:
 			raise AwlSimError("SET_OPT: Value is not a boolean")
 
+	def getFloatValue(self):
+		try:
+			return float(self.value)
+		except ValueError as e:
+			raise AwlSimError("SET_OPT: Value is not a float")
+
 	def toBytes(self):
 		try:
 			payload = self.packString(self.name)
@@ -323,7 +332,7 @@ class AwlSimMessage_CPUDUMP(AwlSimMessage):
 		return cls(dumpText)
 
 class AwlSimMessage_MAINTREQ(AwlSimMessage):
-	plStruct = struct.Struct(">H")
+	plStruct = struct.Struct(str(">H"))
 
 	def __init__(self, requestType):
 		AwlSimMessage.__init__(self, AwlSimMessage.MSG_ID_MAINTREQ)
@@ -346,7 +355,7 @@ class AwlSimMessage_GET_CPUSPECS(AwlSimMessage):
 		AwlSimMessage.__init__(self, AwlSimMessage.MSG_ID_GET_CPUSPECS)
 
 class AwlSimMessage_CPUSPECS(AwlSimMessage):
-	plStruct = struct.Struct(">32I")
+	plStruct = struct.Struct(str(">32I"))
 
 	def __init__(self, cpuspecs):
 		AwlSimMessage.__init__(self, AwlSimMessage.MSG_ID_CPUSPECS)
@@ -390,7 +399,7 @@ class AwlSimMessage_REQ_MEMORY(AwlSimMessage):
 	# Payload header struct:
 	#	flags (32 bit)
 	#	repetition factor (32 bit)
-	plHdrStruct = struct.Struct(">II")
+	plHdrStruct = struct.Struct(str(">II"))
 
 	# Payload memory area struct:
 	#	memType (8 bit)
@@ -398,7 +407,7 @@ class AwlSimMessage_REQ_MEMORY(AwlSimMessage):
 	#	index (16 bit)
 	#	start (32 bit)
 	#	length (32 bit)
-	plAreaStruct = struct.Struct(">BBHII")
+	plAreaStruct = struct.Struct(str(">BBHII"))
 
 	# Flags
 	FLG_SYNC	= 1 << 0 # Synchronous. Returns a REPLY when finished.
@@ -440,7 +449,7 @@ class AwlSimMessage_REQ_MEMORY(AwlSimMessage):
 class AwlSimMessage_MEMORY(AwlSimMessage):
 	# Payload header struct:
 	#	flags (32 bit)
-	plHdrStruct = struct.Struct(">I")
+	plHdrStruct = struct.Struct(str(">I"))
 
 	# Payload memory area struct:
 	#	memType (8 bit)
@@ -449,7 +458,7 @@ class AwlSimMessage_MEMORY(AwlSimMessage):
 	#	start (32 bit)
 	#	length (32 bit)
 	#	the actual binary data (variable length)
-	plAreaStruct = struct.Struct(">BBHII")
+	plAreaStruct = struct.Struct(str(">BBHII"))
 
 	# Flags
 	FLG_SYNC	= 1 << 0 # Synchronous. Returns a REPLY when finished.
@@ -529,6 +538,18 @@ class AwlSimMessageTransceiver(object):
 
 		self.sock.setblocking(False)
 
+	def shutdown(self):
+		if self.sock:
+			try:
+				self.sock.shutdown(socket.SHUT_RDWR)
+			except socket.error as e:
+				pass
+			try:
+				self.sock.close()
+			except socket.error as e:
+				pass
+			self.sock = None
+
 	def send(self, msg):
 		msg.seq = self.txSeqCount
 		self.txSeqCount = (self.txSeqCount + 1) & 0xFFFF
