@@ -19,11 +19,14 @@
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
 
-from awlsim.coreserver.messages import *
+from __future__ import division, absolute_import, print_function, unicode_literals
+from awlsim.core.compat import *
 
-from awlsim.main import *
-from awlsim.parser import *
-from awlsim.cpuspecs import *
+from awlsim.core.main import *
+from awlsim.core.parser import *
+from awlsim.core.cpuspecs import *
+
+from awlsim.coreserver.messages import *
 
 import sys
 import os
@@ -42,11 +45,11 @@ class AwlSimServer(object):
 
 	ENV_MAGIC	= "AWLSIM_CORESERVER_MAGIC"
 
-	enum.start
-	STATE_INIT	= enum.item
-	STATE_RUN	= enum.item
-	STATE_EXIT	= enum.item
-	enum.end
+	EnumGen.start
+	STATE_INIT	= EnumGen.item
+	STATE_RUN	= EnumGen.item
+	STATE_EXIT	= EnumGen.item
+	EnumGen.end
 
 	class Client(object):
 		"""Client information."""
@@ -156,7 +159,7 @@ class AwlSimServer(object):
 
 	def __rebuildSelectReadList(self):
 		rlist = [ self.socket ]
-		rlist.extend(client.socket for client in self.clients)
+		rlist.extend(client.transceiver.sock for client in self.clients)
 		self.__selectRlist = rlist
 
 	def __cpuBlockExitCallback(self, userData):
@@ -238,6 +241,8 @@ class AwlSimServer(object):
 			else:
 				client.nextDump = None
 			self.__updateCpuBlockExitCallback()
+		elif msg.name == "cycle_time_limit":
+			self.sim.cpu.setCycleTimeLimit(msg.getFloatValue())
 		else:
 			status = AwlSimMessage_REPLY.STAT_FAIL
 
@@ -392,8 +397,7 @@ class AwlSimServer(object):
 		printInfo("AwlSimServer: Listening on %s (port %d)..." % (host, port))
 		try:
 			family, socktype, proto, canonname, sockaddr =\
-				socket.getaddrinfo(host, port, socket.AF_INET,
-						   0, socket.SOL_TCP)[0]
+				socket.getaddrinfo(host, port, socket.AF_INET, socket.SOCK_STREAM)[0]
 			sock = socket.socket(family, socktype)
 			sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
 			sock.setblocking(False)
@@ -446,14 +450,8 @@ class AwlSimServer(object):
 			self.sim = None
 
 		for client in self.clients:
-			try:
-				client.socket.shutdown(socket.SHUT_RDWR)
-			except socket.error as e:
-				pass
-			try:
-				client.socket.close()
-			except socket.error as e:
-				pass
+			client.transceiver.shutdown()
+			client.transceiver = None
 			client.socket = None
 		self.clients = []
 
